/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * cmd.c - functions which create MSNP4 commands
 * -------------------------------------------------------------------------- */

#include "msntrans.h"

#ifdef HAVE_SSL

#define MD5Init MD5_Init
#define MD5Update MD5_Update
#define MD5Final MD5_Final

#else
#include "md5.h"
#endif

#define MP_ADD(x, y) (mt_packet_add(x,pstrdup(x->p,y)))

mpacket mt_cmd_ver(unsigned long trid)
{
    mpacket ph;

    ph = mt_packet_heap(3,15);

    MP_ADD(ph,"VER");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,"MSNP2");

    return ph;
}

mpacket mt_cmd_inf(unsigned long trid)
{
    mpacket ph;

    ph = mt_packet_heap(2,10);

    MP_ADD(ph,"INF");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));

    return ph;
}

mpacket mt_cmd_usr(unsigned long trid, char *user, char *chalstr)
{
    mpacket ph;

    ph = mt_packet_heap(4,30);

    MP_ADD(ph,"USR");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,user);
    MP_ADD(ph,chalstr);

    return ph;
}

mpacket mt_cmd_usr_I(unsigned long trid, char *user, char *sp)
{
    mpacket ph;

    ph = mt_packet_heap(5,20);

    MP_ADD(ph,"USR");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,sp);
    MP_ADD(ph,"I");
    MP_ADD(ph,user);

    return ph;
}

mpacket mt_cmd_usr_S(unsigned long trid, char *pass, char *chalstr)
{
    mpacket ph;
    spool sp;
    MD5_CTX context;    
    char *chalinfo, md5[3];
    unsigned char digest[16];
    int i;

    ph = mt_packet_heap(5,30);
    sp = spool_new(ph->p);

    MP_ADD(ph,"USR");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,"MD5");
    MP_ADD(ph,"S");

    /* Concat chalstr and the password */
    chalinfo = spools(ph->p,chalstr,pass,ph->p);

    /* Digest it */
    MD5Init(&context);
    MD5Update(&context,chalinfo,strlen(chalinfo) * sizeof(char));
    MD5Final(digest,&context);

    memset(md5,0,sizeof(md5));
    for (i = 0; i < 16; i++)
    {
        snprintf(md5,3,"%02x",digest[i]);
        spool_add(sp,md5);
    }

    mt_packet_add(ph,spool_print(sp));

    return ph;
}

mpacket mt_cmd_syn(unsigned long trid)
{
    mpacket ph;

    ph = mt_packet_heap(3,15);

    MP_ADD(ph,"SYN");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,"0");

    return ph;
}

mpacket mt_cmd_blp(unsigned long trid, char *l)
{
    mpacket ph;

    ph = mt_packet_heap(3,15);

    MP_ADD(ph,"BLP");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,l);

    return ph;
}

mpacket mt_cmd_gtc(unsigned long trid, char *l)
{
    mpacket ph;

    ph = mt_packet_heap(3,15);

    MP_ADD(ph,"GTC");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,l);

    return ph;
}

mpacket mt_cmd_chg(unsigned long trid, char *state)
{
    mpacket ph;

    ph = mt_packet_heap(3,15);

    MP_ADD(ph,"CHG");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,state);

    return ph;
}

mpacket mt_cmd_add(unsigned long trid, char *list, char *user, char *handle)
{
    mpacket ph;

    ph = mt_packet_heap(5,25);

    MP_ADD(ph,"ADD");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,list);
    MP_ADD(ph,user);
    MP_ADD(ph,handle);

    return ph;
}

mpacket mt_cmd_rem(unsigned long trid, char *list, char *user)
{
    mpacket ph;

    ph = mt_packet_heap(3,15);

    MP_ADD(ph,"REM");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,list);
    MP_ADD(ph,user);

    return ph;
}

mpacket mt_cmd_xfr(unsigned long trid, char *server)
{
    mpacket ph;

    ph = mt_packet_heap(3, 20);

    MP_ADD(ph,"XFR");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,server);

    return ph;
}

mpacket mt_cmd_cal(unsigned long trid, char *user)
{
    mpacket ph;

    ph = mt_packet_heap(3,20);

    MP_ADD(ph,"CAL");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,user);

    return ph;
}

mpacket mt_cmd_ans(unsigned long trid, char *user, char *chalstr, char *sid)
{
    mpacket ph;

    ph = mt_packet_heap(5,25);

    MP_ADD(ph,"ANS");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,user);
    MP_ADD(ph,chalstr);
    MP_ADD(ph,sid);

    return ph;
}

mpacket mt_cmd_msg(unsigned long trid, char *ack, char *msg)
{
    mpacket ph;

    ph = mt_packet_heap(5,30);

    MP_ADD(ph,"MSG");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,ack);
    mt_packet_add(ph,mt_ul2str(ph->p,strlen(msg)));
    mt_packet_add(ph,spools(ph->p,"\r\n",msg,ph->p));
    ph->message = 1;

    return ph;
}

mpacket mt_cmd_fnd(unsigned long trid, char *f, char *l, char *c, char *s, char *cn)
{
    mpacket ph;

    ph = mt_packet_heap(7,25);
    MP_ADD(ph,"FND");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,f);
    MP_ADD(ph,l);
    MP_ADD(ph,c);
    MP_ADD(ph,s);
    MP_ADD(ph,cn);

    return ph;
}

mpacket mt_cmd_rea(unsigned long trid, char *user, char *nick)
{
    mpacket ph;

    ph = mt_packet_heap(4,25);
    MP_ADD(ph,"REA");
    mt_packet_add(ph,mt_ul2str(ph->p,trid));
    MP_ADD(ph,user);
    MP_ADD(ph,nick);

    return ph;
}

mpacket mt_cmd_out(void)
{
    mpacket ph;

    ph = mt_packet_heap(1,6);
    MP_ADD(ph,"OUT");

    return ph;
}
