/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * message.c - Handle messages sent to a MSN user from a session.
 * -------------------------------------------------------------------------- */

#include "msntrans.h"

void mt_message(session s, jpacket jp)
{
    muser u;
    sbsession sb;
    jpq jq;
    char *user;

    log_debug(ZONE,"Session[%s], handling message",jid_full(s->id));

    user = mt_jid2mid(jp->p,jp->to);
    if (user == NULL || j_strcmp(user,s->user) == 0 ||
        (xmlnode_get_tag(jp->x,"body") == NULL && xmlnode_get_tag(xmlnode_get_tag(jp->x,"x?xmlns=jabber:iq:event"),"composing") == NULL))
    {
        /* this message is either invalid or not a normal message */
        jutil_error(jp->x,TERROR_BAD);
        mt_deliver(s->ti,jp->x);
        return;
    }

    if (s->connected == 0)
    {
        log_debug(ZONE,"Queued");
        jq = pmalloco(jp->p,sizeof(_jpq));
        jq->jp = jp;
        pth_msgport_put(s->queue,(pth_message_t *)jq);
        return;
    }

    u = (muser) ghash_get(s->muser_tab,user);
    if (u == NULL || (u->state == ustate_fln && u->sb == NULL))
    {
        jutil_error(jp->x,(terror){405,"MSN Cannot send offline messages"});
        mt_deliver(s->ti,jp->x);
        return;
    }

    sb = u->sb;
    if (sb == NULL)
    {
        /* there is no existing SB session so create one */
        sb = mt_sb_create(s,u);
        mt_mpstream_rwrite(s->st,(handle) mt_phandle_sb_xfr,mt_cmd_xfr(s->st->trid,"SB"),(void*) u);
    }
    else if (sb->connected == 1)
    {
        /* SB session is ready for messages */
        mt_message_sbhandler(u,jp);
        return;
    }

    jq = pmalloco(jp->p,sizeof(_jpq));
    jq->jp = jp;
    pth_msgport_put(sb->queue,(pth_message_t *)jq);
}

#define MIME_MSG "MIME-Version: 1.0\r\nContent-Type: text/plain; charset=UTF-8\r\n\r\n"
#define MIME_TYPE "MIME-Version: 1.0\r\nContent-Type: text/x-msmsgscontrol\r\nTypeingUser: "

void mt_message_sbhandler(muser u, jpacket jp)
{
    sbsession sb = u->sb;
    mpstream st = sb->st;
    pool p = jp->p;
    mpacket msg;
    char *body, *thread;

    body = xmlnode_get_tag_data(jp->x,"body");
    thread = xmlnode_get_tag_data(jp->x,"thread");

    if (thread != NULL && j_strcmp(thread,sb->thread) == 0)
        sb->thread = pstrdup(sb->p,thread);

    if (body != NULL)
        msg = mt_cmd_msg(st->trid,"U",spools(p,MIME_MSG,body,p));
    else
        /* it must be a jabber:x:event */
        msg = mt_cmd_msg(st->trid,"U",spools(p,MIME_TYPE,sb->s->user,"\r\n\r\n",p));

    if (mt_mpstream_write(st,msg) == 0)
    {
        jutil_error(jp->x,TERROR_INTERNAL);
        mt_deliver(sb->s->ti,jp->x);
    }
    else
        xmlnode_free(jp->x);
}
