/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * -------------------------------------------------------------------------- */

#include "msntrans.h"

result mt_receive(instance i, dpacket d, void *arg);
result mt_sessions(void *arg);
void mt_shutdown(void *arg);

/* it all starts here */
void msntrans(instance i, xmlnode x)
{
    mti ti;
    pool p = i->p;
    xmlnode config;
    char *server;

    log_debug(ZONE,"MSN Transport, initializing for section '%s'",i->id);

    /* create new msn transport instance */
    ti = pmalloco(p,sizeof(_mti));
    ti->i = i;
    ti->xc = xdb_cache(i);

    config = xdb_get(ti->xc,jid_new(xmlnode_pool(x),"config@-internal"),"jabber:config:msntrans");
    if (config == NULL)
    {
        log_error(ZONE,"MSN Transport, configuration not found");
        return;
    }

    ti->reg = pstrdup(p,xmlnode_get_tag_data(config,"instructions"));
    if (ti->reg == NULL)
    {
        log_error(ZONE,"MSN Transport, no instructions configured");
        xmlnode_free(config);
        return;
    }

    server = xmlnode_get_tag_data(config,"server-ip");
    ti->server = server ? strdup(server) : strdup(DEFAULT_DS);
    ti->sessions = ghash_create(j_atoi(xmlnode_get_tag_data(config,"prime"),509),
                                (KEYHASHFUNC) str_hash_code,(KEYCOMPAREFUNC) j_strcmp);

    ti->vcard = xmlnode_new_tag_pool(p,"vCard");
    xmlnode_put_attrib(ti->vcard,"xmlns",NS_VCARD);
    xmlnode_insert_node(ti->vcard,xmlnode_get_firstchild(xmlnode_get_tag(config,"vCard")));

    ti->last = xmlnode_new_tag_pool(p,"query");
    xmlnode_put_attrib(ti->last,"xmlns",NS_LAST);
    xmlnode_put_attrib(ti->last,"stamp",jutil_timestamp());

    xmlnode_free(config);

    register_phandler(i,o_DELIVER,mt_receive,(void *) ti);
    register_beat(5,mt_sessions,(void *) ti);
    register_shutdown(mt_shutdown,(void *) ti);
}

result mt_receive(instance i, dpacket d, void *arg)
{
    mti ti = (mti) arg;
    jpacket jp;
    session s;

    jp = jpacket_new(d->x);

    if (!jp->from || !jp->from->user || jpacket_subtype(jp) == JPACKET__ERROR || ti->shutdown == 1)
    {
        log_debug(ZONE,"ingoring packet");
        xmlnode_free(jp->x);
        return r_DONE;
    }

    s = (session) ghash_get(ti->sessions,jid_full(jid_user(jp->from)));
    if (s != NULL && s->exit_flag == 0)
    {
        jp->aux1 = (void *) s;
        mtq_send(s->q,jp->p,mt_session_in,(void *) jp);
    }
    else
        mt_unknown(ti,jp);

    return r_DONE;
}

result mt_sessions(void *arg)
{
    mti ti = (mti) arg;
    session cur, prev;
    pool p;

    prev = NULL;
    cur = ti->dead;

    while (cur != NULL)
    {
        log_debug(ZONE,"Session %s, %X %d",jid_full(cur->id),cur,cur->ref);

        if (cur->ref == 0)
        {
            log_debug(ZONE,"Removing session");

            if (prev)
                prev->next = cur->next;
            else
                ti->dead = cur->next;

            p = cur->p;
            cur = cur->next;
            pool_free(p);
            --ti->sessions_count;
        }
        else
        {
            prev = cur;
            cur = cur->next;
        }
    }

    if (ti->shutdown && ti->sessions_count == 0)
        pth_cond_notify(ti->cond,FALSE);

    if (ti->sessions_count)
        log_debug(ZONE,"%d session(s)",ti->sessions_count);

    return r_DONE;
}

int mt_sessions_end(void *arg, const void *key, void *data)
{
    mt_session_end((session) data);
    return 1;
}

void mt_shutdown(void *arg)
{
    mti ti = (mti) arg;
    pth_mutex_t mutex = PTH_MUTEX_INIT;
    pth_cond_t cond = PTH_COND_INIT;

    log_notice(ZONE,"MSN Transport, shutting down");

    ti->shutdown = 1;
    ti->cond = &cond;

    ghash_walk(ti->sessions,mt_sessions_end,NULL);

    /* wait for the sessions to end */
    pth_mutex_acquire(&mutex,FALSE,NULL);
    pth_cond_await(&cond,&mutex,NULL);

    ghash_destroy(ti->sessions);
    free(ti->server);
}
