/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * -------------------------------------------------------------------------- */

#include "jabberd.h"

#define CONN_AT_MAX 6 /* Maximum connection attempts to the Dispatch Server */
#define DEFAULT_DS "64.4.13.57"
//#define PACKET_DEBUG

typedef struct session_st *session, _session;

/* MSN Transport instance */
typedef struct
{
    instance i;
    xdbcache xc;
    HASHTABLE sessions;
    session dead;
    pth_cond_t *cond;
    xmlnode vcard, last;
    char *server, *reg;
    int sessions_count, shutdown;
} *mti, _mti;

typedef struct
{
    void *data1;
    void *data2;
} *smsg, _smsg;

typedef struct mpdata_st
{
    struct mpdata_st *next;
    char *data;
} *mpdata, _mpdata;

typedef struct
{
    pool p;
    mpdata data, cur;
    int c, message;
} *mpacket, _mpacket;

typedef int (*handle)(mpacket ph, void *arg);

typedef struct
{
    handle h;
    void *arg;
    mpacket ph;
    char *part;
} *mparser, _mparser;

typedef struct mphandler_st
{
    pool p;
    handle h;
    unsigned long trid;
    void *arg;
    struct mphandler_st *next;
} *mphandler, _mphandler;

typedef enum { conn_CLOSED, conn_OPEN } conn_state;

typedef struct
{
    pool p;
    mio c;
    session s;
    mphandler phan;
    unsigned long trid;
    handle h;
    void *arg;
    mparser par;
    conn_state state;
    mpacket buf;
} *mpstream, _mpstream;

typedef enum
{
    ustate_nln,
    ustate_fln,
    ustate_bsy,
    ustate_awy,
    ustate_phn,
    ustate_brb,
    ustate_idl,
    ustate_lun
} ustate;

typedef struct sbuser_st *sbuser, _sbuser;

typedef struct
{
    pool p;
    session s;
    mpstream st;
    sbuser users;
    pth_msgport_t queue;
    char *thread;
    int connected, count;
} *sbsession, _sbsession;

typedef struct
{
    pool p;
    ustate state;
    sbsession sb;
    char *m, *handle;
    int rl, fl, al, bl, rl_flag;
} *muser, _muser;

struct sbuser_st
{
    muser u;
    struct sbuser_st *next;
};

typedef enum { stype_normal, stype_register } session_type;

struct session_st
{
    pool p;
    mti ti;
    mtq q;
    pth_msgport_t queue;
    jid id, from;
    char *server;
    session_type type;
    session next;
    ustate state;
    mpstream st;
    HASHTABLE muser_tab;
    ppdb p_db;
    char *pass, *user, *nick;
    int exit_flag, conn_at, connected, ref;
};

typedef struct
{
    pth_message_t head;
    jpacket jp;
} *jpq, _jpq;

void msntrans(instance i, xmlnode x);
void mt_unknown(mti ti, jpacket jp);

session mt_session_create(mti ti, jpacket jp);
void mt_session_connect(session s, char *host, int port);
void mt_session_start(session s);
void mt_session_in(void *arg);
void mt_session_jpacket(session s, jpacket jp);
void mt_session_packet(void *arg);
void mt_session_register(session s);
void mt_session_end(session s);
void mt_session_kill(session s, terror t);

void mt_s10n(session s, jpacket jp);
void mt_presence(session s, jpacket jp);

void mt_message(session s, jpacket jp);
void mt_message_sbhandler(muser u, jpacket jp);

void mt_iq(session s, jpacket jp);
void mt_iq_search_get(session s, jpacket jp);
void mt_iq_search_set(session s, jpacket jp);
void mt_iq_vcard_server(mti ti, jpacket jp);
void mt_iq_browse_server(mti ti, jpacket jp);
void mt_iq_time(mti ti, jpacket jp);
void mt_iq_version(mti ti, jpacket jp);

muser mt_muser(session s, char *m);
int mt_muser_lst_add(session s, mpacket ph);
void mt_muser_lst_end(session s, pool p);
void mt_muser_roster(session s, pool p, char *user, int adding);
int mt_muser_cleanup_walker(void *arg, const void *key, void *data);

sbsession mt_sb_create(session s, muser u);
mpstream mt_sb_connect(sbsession sb, char *host, int port);
void mt_sb_add(sbsession sb, muser u);
void mt_sb_bye(sbsession sb, char *m);
void mt_sb_close(sbsession sb);

mpstream mt_mpstream_connect(session s, char *host, int port, handle h, void *arg);
int mt_mpstream_write(mpstream st, mpacket ph);
void mt_mpstream_register(mpstream st, handle h, unsigned long trid, void *arg);
void mt_mpstream_dispatch(mpstream st, mpacket ph);

int mt_phandle_default(mpacket ph, void *arg);
int mt_phandle_ver(mpacket ph, void *arg);
int mt_phandle_inf(mpacket ph, void *arg);
int mt_phandle_usr_i(mpacket ph, void *arg);
int mt_phandle_usr_s(mpacket ph, void *arg);
int mt_phandle_syn(mpacket ph, void *arg);
int mt_phandle_chg(mpacket ph, void *arg);
int mt_phandle_add(mpacket ph, void *arg);
int mt_phandle_rem(mpacket ph, void *arg);
int mt_phandle_fnd(mpacket ph, void *arg);
void mt_phandle_xfr(mpacket ph, session s);
void mt_phandle_rng(mpacket ph, session s);
void mt_phandle_iln(mpacket ph, session s);
void mt_phandle_nln(mpacket ph, session s);
void mt_phandle_fln(mpacket ph, session s);
void mt_phandle_msg(mpacket ph, session s);

int mt_phandle_sb_default(mpacket ph, void *arg);
int mt_phandle_sb_xfr(mpacket ph, void *arg);
int mt_phandle_sb_usr(mpacket ph, void *arg);
int mt_phandle_sb_ans(mpacket ph, void *arg);
void mt_phandle_sb_msg(mpacket ph, sbsession sb);

mpacket mt_cmd_ver(unsigned long trid);
mpacket mt_cmd_inf(unsigned long trid);
mpacket mt_cmd_usr(unsigned long trid, char *user, char *chalstr);
mpacket mt_cmd_usr_I(unsigned long trid, char *user, char *sp);
mpacket mt_cmd_usr_S(unsigned long trid, char *pass, char *chalstr);
mpacket mt_cmd_syn(unsigned long trid);
mpacket mt_cmd_blp(unsigned long trid, char *l);
mpacket mt_cmd_gtc(unsigned long trid, char *l);
mpacket mt_cmd_chg(unsigned long trid, char *state);
mpacket mt_cmd_add(unsigned long trid, char *list, char *user, char *handle);
mpacket mt_cmd_rem(unsigned long trid, char *list, char *user);
mpacket mt_cmd_xfr(unsigned long trid, char *server);
mpacket mt_cmd_cal(unsigned long trid, char *user);
mpacket mt_cmd_ans(unsigned long trid, char *user, char *chalstr, char *sid);
mpacket mt_cmd_msg(unsigned long trid, char *ack, char *msg);
mpacket mt_cmd_fnd(unsigned long trid, char *f, char *l, char *c, char *s, char *cn);
mpacket mt_cmd_rea(unsigned long trid, char *user, char *nick);
mpacket mt_cmd_out(void);

mparser mt_parser_new(pool p, handle h, void *arg);
mpacket mt_packet_heap(int num_param, int s);
mpacket mt_packet_new(void);
void mt_packet_add(mpacket ph, char *data);
char *mt_packet_data(mpacket ph, int i);
char *mt_packet_str(mpacket ph);
void mt_packet_parse(mparser par, char *packet);

char *mt_safe_nick(pool p, char *nick);
char *mt_safe_user(pool p, char *user);
xmlnode mt_presnew(int type, char *to, char *from, char *show);
xmlnode mt_jmessage(int type, char *to, char *from, char *body, char *thread);
char *mt_jid2mid(pool p, jid id);
jid mt_mid2jid(pool p, char *user, char *server);
xmlnode mt_user_presnew(session s, char *m, ustate state);
char *mt_ustate2state(pool p, ustate state);
char *mt_show2state(pool p, char *show);
ustate mt_state2ustate(char *state);
char *mt_fnd2cdata(char *fnd);
char *mt_ul2str(pool p, unsigned long ul);
int mt_parse_msg(mpacket ph, char **msg, char **ctype);
char *mt_str_to_UTF8(pool p, unsigned char *in);
jid mt_xdb_id(pool p, jid id, char *server);

#define mt_mpstream_rwrite(s,h,m,a) if(mt_mpstream_write(s,m)) \
  mt_mpstream_register(s,h,s->trid - 1,a)
#define mt_deliver(ti, x) deliver(dpacket_new(x),ti->i)
