/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * -------------------------------------------------------------------------- */

#include "msntrans.h"

muser mt_muser(session s, char *m)
{
    muser u;
    pool p;

    if (m == NULL)
        return NULL;

    u = (muser) ghash_get(s->muser_tab,m);
    if (u == NULL)
    {
        log_debug(ZONE,"muser new entry, %s",m);

        p = pool_new();
        u = pmalloco(p,sizeof(_muser));
        u->p = p;
        u->m = pstrdup(p,m);
        u->state = ustate_fln;

        ghash_put(s->muser_tab,u->m,u);
    }

    return u;
}

int mt_muser_lst_add(session s, mpacket ph)
{
    muser u;
    char *list;
    int total, item;

    list = mt_packet_data(ph,2);   /* Get the list type */
    item = j_atoi(mt_packet_data(ph,4),0);
    total = j_atoi(mt_packet_data(ph,5),0);

    u = mt_muser(s,mt_packet_data(ph,6));
    if (u != NULL && list != NULL)
    {
        if (j_strcmp(list,"FL") == 0)
            u->fl = 1;
        else if (j_strcmp(list,"AL") == 0)
            u->al = 1;
        else if (j_strcmp(list,"BL") == 0)
            u->bl = 1;
        else if (j_strcmp(list,"RL") == 0)
            u->rl = 1;
    }

    /* We know when we have received all entries in a list when the item# is equal to the total */
    if (item != total || j_strcmp(list,"RL") != 0)
        return 0;

    /* all the lists have been received */
    mt_muser_lst_end(s,ph->p);
    mt_mpstream_rwrite(s->st,mt_phandle_chg,mt_cmd_chg(s->st->trid,mt_ustate2state(ph->p,s->state)),s);
    return 1;
}

int mt_muser_lst_walker(void *arg, const void *key, void *data)
{
    session s = (session) arg;
    muser u = (muser) data;
    xmlnode x;
    pool p;

    if (u->rl_flag == 0 && u->rl == 1 && u->bl == 0)
    {
        x = mt_presnew(JPACKET__SUBSCRIBE,jid_full(s->id),NULL,NULL);

        p = xmlnode_pool(x);
        mt_muser_roster(s,p,u->m,1);

        xmlnode_put_attrib(x,"from",jid_full(mt_mid2jid(p,u->m,s->server)));
        mt_deliver(s->ti,x);
    }
    else if (u->rl_flag == 1 && u->rl == 0)
    {
        /* the user has been removed from the RL*/
        x = mt_presnew(JPACKET__UNSUBSCRIBE,jid_full(s->id),NULL,NULL);

        p = xmlnode_pool(x);
        mt_muser_roster(s,p,u->m,0);

        xmlnode_put_attrib(x,"from",jid_full(mt_mid2jid(p,u->m,s->server)));
        mt_deliver(s->ti,x);
    }

    return 1;
}

void mt_muser_lst_end(session s, pool p)
{
    mti ti = s->ti;
    xmlnode roster, cur;
    muser u;

    roster = xdb_get(ti->xc,mt_xdb_id(p,s->id,s->server),NS_ROSTER);
    if (roster != NULL)
    {
        for (cur = xmlnode_get_firstchild(roster); cur != NULL; cur = xmlnode_get_nextsibling(cur))
            if (xmlnode_get_type(cur) == NTYPE_TAG && (u = mt_muser(s,xmlnode_get_attrib(cur,"jid"))) != NULL)
                u->rl_flag = 1;

        xmlnode_free(roster);
    }

    ghash_walk(s->muser_tab,mt_muser_lst_walker,(void *) s);
}

void mt_muser_roster(session s, pool p, char *user, int adding)
{
    mti ti = s->ti;
    xmlnode roster, item = NULL;
    jid id;

    id = mt_xdb_id(p,s->id,s->server);
    roster = xdb_get(ti->xc,id,NS_ROSTER);

    if (roster != NULL)
        item = xmlnode_get_tag(roster,spools(p,"?jid=",user,p));

    if ((item == NULL && adding == 0) || (item != NULL && adding == 1))
    {
        if (roster != NULL)
            xmlnode_free(roster);
        return;
    }

    if (adding)
    {
        if (roster == NULL)
        {
            roster = xmlnode_new_tag("query");
            xmlnode_put_attrib(roster,"xmlns",NS_ROSTER);
        }

        item = xmlnode_insert_tag(roster,"item");
        xmlnode_put_attrib(item,"jid",user);
        xmlnode_put_attrib(item,"subscription","from");
    }
    else
        xmlnode_hide(item);

    xdb_set(ti->xc,id,NS_ROSTER,roster);
}

int mt_muser_cleanup_walker(void *arg, const void *key, void *data)
{
    session s = (session) arg;
    muser u = (muser) data;
    xmlnode x;

    if (u->state != ustate_fln)
    {
        x = mt_presnew(JPACKET__UNAVAILABLE,jid_full(s->id),NULL,NULL);
        xmlnode_put_attrib(x,"from",jid_full(mt_mid2jid(xmlnode_pool(x),u->m,s->server)));
        mt_deliver(s->ti,x);
    }

    if (u->sb != NULL)
        mt_sb_close(u->sb);

    ghash_remove(s->muser_tab,u->m);
    pool_free(u->p);

    return 1;
}
