/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * -------------------------------------------------------------------------- */

#include "msntrans.h"

void mt_parser_cleanup(void *arg)
{
    mparser par = (mparser) arg;

    if (par->part != NULL)
        free(par->part);

    if (par->ph != NULL)
        pool_free(par->ph->p);
}

mparser mt_parser_new(pool p, handle h, void *arg)
{
    mparser par;

    par = pmalloco(p,sizeof(_mparser));
    par->h = h;
    par->arg = arg;

    pool_cleanup(p,mt_parser_cleanup,par);

    return par;
}

mpacket mt_packet_heap(int num_param, int s)
{
    pool p;
    mpacket ph;
    int len;

    len = num_param * sizeof(_mpdata);
    len += s * sizeof(char);

    p = pool_heap(len);
    ph = (mpacket) pmalloco(p,sizeof(_mpacket));
    ph->p = p;

    return ph;
}

mpacket mt_packet_new(void)
{
    mpacket ph;
    pool p;

    p = pool_new();
    ph = (mpacket) pmalloco(p,sizeof(_mpacket));
    ph->p = p;

    return ph;
}

void mt_packet_add(mpacket ph, char *data)
{
    mpdata d, cur;

    d = pmalloco(ph->p,sizeof(_mpdata));
    d->data = data;

    if (ph->data != NULL)
    {
        if (ph->cur != NULL)
            cur = ph->cur;
        else
            cur = ph->data;

        cur->next = d;
        ph->cur = cur->next;
    }
    else
        ph->data = d;
}

char *mt_packet_data(mpacket ph, int i)
{
    mpdata cur;
    int c = ph->c;

    if (i < c || ph->cur == NULL)
    {
        c = 0;
        cur = ph->data;
    }
    else
        cur = ph->cur;

    while (cur != NULL)
    {
        if (c == i)
        {
            ph->cur = cur;
            ph->c = c;
            return cur->data;
        }
        cur = cur->next;
        c++;
    }
    return NULL;
}

char *mt_packet_str(mpacket ph)
{
    mpdata cur;
    spool sp;

    sp = spool_new(ph->p);
    for (cur = ph->data; cur != NULL; cur = cur->next) /* loop the parameters */
    {
        spool_add(sp,cur->data);

        if (cur->next != NULL)
            spool_add(sp," ");
        else if (ph->message == 0)
            spool_add(sp,"\r\n");
    }

    return spool_print(sp);
}

void mt_packet_parse(mparser par, char *packet)
{
    mpacket ph;
    size_t sz, len;
    char *part, *dest, c;
    int i;

    if (par->ph != NULL)
    {
        ph = par->ph;
        par->ph = NULL;
    }
    else
        ph = mt_packet_new();

    if (par->part != NULL)
    {
        packet = spools(ph->p,par->part,packet,ph->p);
        free(par->part);
        par->part = NULL;
    }

    sz = strlen(packet);
    part = packet;

    for (i = 0; i < sz; i++)
    {
        c = packet[i];

        if (c == ' ') {

            if (part == NULL)
            {
                /* parse error */
                log_error(NULL,"NULL part, %s",packet);
                (par->h) (NULL,par->arg);
                if (ph) pool_free(ph->p);
                return;
            }

            if (ph == NULL)
                ph = mt_packet_new();

            *(packet + i) = '\0';

            len = strlen(part);
            dest = pmalloco(ph->p,len + 1);
            mt_packet_add(ph,strncpy(dest,part,len));
            part = NULL;

        } else if (c == '\r') {

            /* check for a split packet */
            if (i + 1 < sz && packet[i + 1] != '\n')
                break;

            if (part == NULL || ph == NULL)
            {
                /* parse error */
                log_error(NULL,"NULL %d, %s",part == NULL,packet);
                (par->h) (NULL,par->arg);
                if (ph) pool_free(ph->p);
                return;
            }

            /* we've reached the end of current packet */
            *(packet + i) = '\0'; /* NULL terminate the last parameter */
            ++i;

            len = strlen(part);
            dest = pmalloco(ph->p,len + 1);
            mt_packet_add(ph,strncpy(dest,part,len));
            ph->cur = NULL;

            if (j_strcmp(mt_packet_data(ph,0),"MSG") == 0)
            {
                char *msg;
                int l;

                l = atol(mt_packet_data(ph,3));
                if (i + l < sz)
                {
                    msg = pmalloco(ph->p,l + 1 * sizeof(char));
                    mt_packet_add(ph,strncpy(msg,packet + i + 1,l));
                    ph->message = 1;
                    i += l;
                }
                else
                {
                    part = packet + i + 1;
                    break;
                }
            }

            if ((par->h) (ph,par->arg) == 0)  /* stop parsing */
                return;

            part = NULL;
            ph = NULL;

        } else if (part == NULL) {

            /* if part is NULL, this must be the start of a new parameter */
            part = packet + i;
        }
    }

    if (part != NULL)
    {
        /* we only have part of this parameter */
        log_debug(ZONE,"Saving part");
        par->part = strdup(part);
    }

    if (ph != NULL)
    {
        /* we only have part of this packet */
        log_debug(ZONE,"Saving packet");
        par->ph = ph;
    }
}
