/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * phandle.c - this is where all the processing of MSN commands happens 
 * -------------------------------------------------------------------------- */

#include "msntrans.h"

int mt_phandle_default(mpacket ph, void *arg)
{
    session s = (session) arg;
    char *cmd;

    cmd = mt_packet_data(ph,0);

    if (j_strcmp(cmd,"NLN") == 0)
        mt_phandle_nln(ph,s);
    else if (j_strcmp(cmd,"FLN") == 0)
        mt_phandle_fln(ph,s);
    else if (j_strcmp(cmd,"ADD") == 0)
        mt_phandle_add(ph,arg);
    else if (j_strcmp(cmd,"REM") == 0)
        mt_phandle_rem(ph,arg);
    else if (j_strcmp(cmd,"RNG") == 0)
        mt_phandle_rng(ph,s);
    else if (j_strcmp(cmd,"XFR") == 0)
        mt_phandle_xfr(ph,s);
    else if (j_strcmp(cmd,"MSG") == 0)
        mt_phandle_msg(ph,s);
    else if (j_strcmp(cmd,"OUT") == 0)
        mt_session_end(s);
    else if (j_strcmp(cmd,"ILN") == 0)
        mt_phandle_iln(ph,s);
    else
        return 1;

    return 0;
}

int mt_phandle_ver(mpacket ph, void *arg)
{
    session s = (session) arg;

    if (j_strcmp(mt_packet_data(ph,0),"VER") != 0)
    {
        /* The server is probably down */
        log_debug(ZONE,"Session[%s], Unknown command %s, restarting",jid_full(s->id),mt_packet_str(ph));
        mt_session_start(s);
    }
    else
        mt_mpstream_rwrite(s->st,mt_phandle_inf,mt_cmd_inf(s->st->trid),(void *) s);

    return 1;
}

int mt_phandle_inf(mpacket ph, void *arg)
{
    session s = (session) arg;
    mt_mpstream_rwrite(s->st,mt_phandle_usr_i,mt_cmd_usr_I(s->st->trid,s->user,"MD5"),(void *) s);
    return 1;
}

int mt_phandle_usr_i(mpacket ph, void *arg)
{
    session s = (session) arg;
    char *cmd, *chalstr;

    cmd = mt_packet_data(ph,0);

    if (j_strcmp(cmd,"USR") == 0)
    {
        chalstr = mt_packet_data(ph,4);
        mt_mpstream_rwrite(s->st,mt_phandle_usr_s,mt_cmd_usr_S(s->st->trid,s->pass,chalstr),s);
    }
    else if (j_strcmp(cmd,"XFR") == 0)
        mt_phandle_xfr(ph,s);
    else if (j_atoi(cmd,0) == 911)
        mt_session_kill(s,(terror) {406,"Invalid Username"});
    else
        mt_session_kill(s,TERROR_EXTERNAL);

    return 1;
}
    
int mt_phandle_usr_s(mpacket ph, void *arg)
{
    session s = (session) arg;
    char *cmd;

    cmd = mt_packet_data(ph,0);

    if (j_strcmp(cmd,"USR") == 0)
    {
        /* auth was successful */
        if (s->type == stype_register)
            mt_session_register(s);

        if (j_strcmp(mt_packet_data(ph,4),s->nick) == 0)
            s->nick = NULL;

        mt_mpstream_rwrite(s->st,mt_phandle_syn,mt_cmd_syn(s->st->trid),(void *) s);
    }
    else if (j_atoi(cmd,0) == 911)
        mt_session_kill(s,TERROR_AUTH);
    else
        mt_session_start(s);  /* some other wierd error, try again */

    return 1;
}

int mt_phandle_syn(mpacket ph, void *arg)
{
    session s = (session) arg;
    char *cmd, *state;

    cmd = mt_packet_data(ph,0);

    if (j_strcmp(cmd,"LST") == 0)
    {
        return mt_muser_lst_add(s,ph);
    }
    else if (j_strcmp(cmd,"SYN") == 0 && j_atoi(mt_packet_data(ph,2),0) == 0)
    {
        /* No LST commands will be received since the Seq number is zero */
        state = mt_ustate2state(ph->p,s->state);
        mt_mpstream_rwrite(s->st,mt_phandle_chg,mt_cmd_chg(s->st->trid,state),(void *) s);
        return 1;
    }

    return 0;
}

int mt_phandle_chg(mpacket ph, void *arg)
{
    session s = (session) arg;
    char *cmd;

    cmd = mt_packet_data(ph,0);

    if (j_strcmp(cmd,"CHG") == 0)
    {
        if (s->connected == 0)
        {
            xmlnode pres;
            jpq jq;

            pres = mt_user_presnew(s,NULL,s->state);
            xmlnode_insert_cdata(xmlnode_insert_tag(pres,"status"),"Connected",-1);
            xmlnode_put_attrib(pres,"from",jid_full(s->from));
            mt_deliver(s->ti,pres);

            if (s->nick != NULL)
                mt_mpstream_write(s->st,mt_cmd_rea(s->st->trid,s->user,s->nick));

            s->connected = 1;

            while ((jq = (jpq) pth_msgport_get(s->queue)) != NULL)
                mt_session_jpacket(s,jq->jp);
        }
    }
    else if (j_strcmp(cmd,"ILN") == 0)
    {
        mt_phandle_iln(ph,s);
        return 0;
    }
    else
        log_debug(ZONE,"Session[%s], Unknown command, %d",jid_full(s->id),cmd);

    return 1;
}


int mt_phandle_add(mpacket ph, void *arg)
{
    session s = (session) arg;
    muser u;
    jid from;
    char *user, *list, *cmd;

    cmd = mt_packet_data(ph,0);
    if (j_strcmp(cmd,"ADD") != 0)
    {
        if (j_strcmp(cmd,"ILN") == 0)
        {
            mt_phandle_iln(ph,arg);
            return 0;
        }
        log_debug(ZONE,"Session[%s], error adding user",jid_full(s->id));
        return 1;
    }

    list = mt_packet_data(ph,2);
    user = mt_packet_data(ph,4);

    if ((u = mt_muser(s,user)) == NULL) return 1;

    if (j_strcmp(list,"RL") == 0)
    {
        u->rl = 1;
        mt_muser_roster(s,ph->p,u->m,1);
        from = mt_mid2jid(ph->p,user,s->server);
        mt_deliver(s->ti,mt_presnew(JPACKET__SUBSCRIBE,jid_full(s->id),jid_full(from),NULL));
    }
    else if (j_strcmp(list,"BL") == 0)
    {
        u->bl = 1;
        from = mt_mid2jid(ph->p,user,s->server);
        mt_deliver(s->ti,mt_presnew(JPACKET__UNAVAILABLE,jid_full(s->id),jid_full(from),NULL));
    }
    else if (j_strcmp(list,"AL") == 0)
        u->al = 1;
    else if (j_strcmp(list,"FL") == 0)
        u->fl = 1;

    return 1;
}

int mt_phandle_rem(mpacket ph, void *arg)
{
    session s = (session) arg;
    muser u;
    jid from;
    char *user, *list, *cmd;

    cmd = mt_packet_data(ph,0);
    if (j_strcmp(cmd,"REM") != 0)
    {
        log_debug(ZONE,"Session[%s], removing user",jid_full(s->id));
        return 1;
    }

    list = mt_packet_data(ph,2);
    user = mt_packet_data(ph,4);

    u = mt_muser(s,user);
    if (u == NULL)
    {
        log_debug(ZONE,"Error removing user, %s",mt_packet_str(ph));
        return 1;
    }

    if (j_strcmp(list,"RL") == 0)
    {
        u->rl = 0;
        mt_muser_roster(s,ph->p,u->m,0);
        from = mt_mid2jid(ph->p,user,s->server);
        mt_deliver(s->ti,mt_presnew(JPACKET__UNSUBSCRIBE,jid_full(s->id),jid_full(from),NULL));
    }
    else if (j_strcmp(list,"FL") == 0)
    {
        u->fl = 0;
        if (u->state != ustate_fln)
        {
            u->state = ustate_fln;
            from = mt_mid2jid(ph->p,user,s->server);
            mt_deliver(s->ti,mt_presnew(JPACKET__UNAVAILABLE,jid_full(s->id),jid_full(from),NULL));
        }
    }
    else if (j_strcmp(list,"AL") == 0)
        u->al = 0;
    else if (j_strcmp(list,"BL") == 0)
        u->bl = 0;

    return 1;
}

int mt_phandle_fnd(mpacket ph, void *arg)
{
    xmlnode data = (xmlnode) arg;
    xmlnode q, item;
    session s;
    char *f, *l, *city, *state, *country;

    s = (session) xmlnode_get_vattrib(data,"s");

    if (j_strcmp(mt_packet_data(ph,0),"FND") != 0)
    {
        log_debug(ZONE,"Session[%s], Unknown search result",jid_full(s->id));
        xmlnode_hide_attrib(data,"s");
        mt_deliver(s->ti,data);
        return 1;
    }

    f = mt_fnd2cdata(mt_packet_data(ph,4));
    l = mt_fnd2cdata(mt_packet_data(ph,5));
    city = mt_fnd2cdata(mt_packet_data(ph,6));
    state = mt_fnd2cdata(mt_packet_data(ph,7));
    country = mt_fnd2cdata(mt_packet_data(ph,8));

    q = xmlnode_insert_tag(data,"query");
    xmlnode_put_attrib(q,"xmlns",NS_SEARCH);

    item = xmlnode_insert_tag(q,"item");
    xmlnode_put_attrib(item,"jid","none");
    xmlnode_insert_cdata(xmlnode_insert_tag(item,"first"),f,-1);
    xmlnode_insert_cdata(xmlnode_insert_tag(item,"last"),l,-1);

    if (j_atoi(mt_packet_data(ph,2),0) != j_atoi(mt_packet_data(ph,3),0))
        return 0;

    xmlnode_hide_attrib(data,"s");
    mt_deliver(s->ti,data);

    return 1;
}

void mt_phandle_xfr(mpacket ph, session s)
{
    char *host, *port;

    if (j_strcmp(mt_packet_data(ph,2),"NS"))
    {
        log_debug(ZONE,"Session[%s], NS XFR Error, %s",jid_full(s->id),mt_packet_data(ph,2));
        return;
    }

    host = mt_packet_data(ph,3);
    port = strchr(host,':');
    *port = '\0';
    ++port;

    mt_session_connect(s,host,j_atoi(port,1863));
    if (strcmp(host,s->ti->server) != 0)
    {
        free(s->ti->server);
        s->ti->server = strdup(host);
    }
}

void mt_phandle_rng(mpacket ph, session s)
{
    mpstream st;
    muser u;
    sbsession sb;
    char *host, *port, *sid, *chalstr;

    u = mt_muser(s,mt_packet_data(ph,5));
    if (u == NULL || u->sb != NULL)
        return;

    host = mt_packet_data(ph,2);
    port = strchr(host,':');
    *port = '\0';
    port++;

    sb = mt_sb_create(s,u);
    st = mt_sb_connect(sb,host,j_atoi(port,1863));

    sid = mt_packet_data(ph,1);
    chalstr = mt_packet_data(ph,4);
    mt_mpstream_rwrite(st,mt_phandle_sb_ans,mt_cmd_ans(st->trid,s->user,chalstr,sid),u);
}

void mt_phandle_iln(mpacket ph, session s)
{
    muser u;
    char *state;

    state = mt_packet_data(ph,2);
    u = mt_muser(s,mt_packet_data(ph,3));
    u->handle = pstrdup(u->p,mt_packet_data(ph,4));
    u->state = mt_state2ustate(state);

    mt_deliver(s->ti,mt_user_presnew(s,u->m,u->state));
}

void mt_phandle_nln(mpacket ph, session s)
{
    muser u;
    char *state;

    state = mt_packet_data(ph,1);
    u = mt_muser(s,mt_packet_data(ph,2));
    u->handle = pstrdup(u->p,mt_packet_data(ph,3));
    u->state = mt_state2ustate(state);

    mt_deliver(s->ti,mt_user_presnew(s,u->m,u->state));
}

void mt_phandle_fln(mpacket ph, session s)
{
    muser u;

    u = mt_muser(s,mt_packet_data(ph,1));
    u->state = ustate_fln;

    mt_deliver(s->ti,mt_user_presnew(s,u->m,u->state));
}

void mt_phandle_msg(mpacket ph, session s)
{

    xmlnode x, oob;
    char *msg, *ctype, *ptr;

    if (mt_parse_msg(ph,&msg,&ctype) == 0)
    {
        log_debug(ZONE,"Session failed to parse messsage");
        return;
    }

    /* this message is a Hotmail inbox notification */
    if ((strncmp(ctype,"text/x-msmsgsinitialemailnotification",37 * sizeof(char)) != 0) &&
        (strncmp(ctype,"application/x-msmsgsemailnotification",37 * sizeof(char)) != 0))
            return;

    /* cut off the junk at the end */
    if ((ptr = strstr(msg,"Inbox-URL")) != NULL) *ptr = '\0';

    x = mt_jmessage(JPACKET__HEADLINE,jid_full(s->id),s->server,msg,NULL);
    xmlnode_insert_cdata(xmlnode_insert_tag(x,"subject"),"Hotmail",-1);

    oob = xmlnode_insert_tag(x,"x");
    xmlnode_put_attrib(oob,"xmlns","jabber:x:oob");
    xmlnode_insert_cdata(xmlnode_insert_tag(oob,"url"),"http://www.hotmail.com/cgi-bin/folders",-1);
    xmlnode_insert_cdata(xmlnode_insert_tag(oob,"desc"),"Login to your Hotmail e-mail account",-1);

    mt_deliver(s->ti,x);
}

int mt_phandle_sb_default(mpacket ph, void *arg)
{
    sbsession sb = (sbsession) arg;
    muser u;
    char *cmd;

    cmd = mt_packet_data(ph,0);

    if (j_strcmp(cmd,"MSG") == 0)
    {
        mt_phandle_sb_msg(ph,sb);
    }
    else if (j_strcmp(cmd,"JOI") == 0)
    {
        session s = sb->s;

        u = mt_muser(s,mt_packet_data(ph,1));
        mt_sb_add(sb,u);

        if (sb->connected == 0)
        {
            jpq jq;

            sb->connected = 1;

            while ((jq = (jpq) pth_msgport_get(sb->queue)) != NULL)
                mt_message_sbhandler(u,jq->jp);
        }
    }
    else if (j_strcmp(cmd,"BYE") == 0)
    {
        mt_sb_bye(sb,mt_packet_data(ph,1));
    }
    else if (j_atoi(cmd,0) != 0)
    {
        log_debug(ZONE,"SB Error, %s",cmd);
        mt_sb_close(sb);
    }
    else
        return 1;

    return 0;
}

int mt_phandle_sb_xfr(mpacket ph, void *arg)
{
    muser u = (muser) arg;
    sbsession sb = u->sb;
    mpstream st;
    char *host, *port, *chalstr;

    if (j_strcmp(mt_packet_data(ph,2),"SB"))
    {
        log_debug(ZONE,"SB session, XFR error, %s",mt_packet_data(ph,0));
        mt_sb_close(sb);
        return 1;
    }

    host = mt_packet_data(ph,3);
    port = strstr(host,":");
    *port = '\0';
    ++port;

    st = mt_sb_connect(sb,host,j_atoi(port,1863));
    chalstr = mt_packet_data(ph,5);
    mt_mpstream_rwrite(st,mt_phandle_sb_usr,mt_cmd_usr(st->trid,sb->s->user,chalstr),u);

    return 1;
}

int mt_phandle_sb_usr(mpacket ph, void *arg)
{
    muser u = (muser) arg;
    sbsession sb = u->sb;
    char *cmd;

    cmd = mt_packet_data(ph,0);
    if (j_strcmp(cmd,"USR") != 0)
    {
        log_debug(ZONE,"SB session, USR error, %s",cmd);
        mt_sb_close(sb);
    }
    else
        mt_mpstream_write(sb->st,mt_cmd_cal(sb->st->trid,u->m));

    return 1;
}

int mt_phandle_sb_ans(mpacket ph, void *arg)
{
    muser u = (muser) arg;
    sbsession sb = u->sb;
    char *cmd, *user;

    cmd = mt_packet_data(ph,0);
    if (j_strcmp(cmd,"IRO") == 0)
    {
        user = mt_packet_data(ph,1);
        if (j_strcmp(user,u->m) != 0)
            mt_sb_add(sb,mt_muser(sb->s,user));

        return 0;
    }

    if (j_strcmp(cmd,"ANS") == 0)
    {
        jpq jq;

        sb->connected = 1;
        while ((jq = (jpq) pth_msgport_get(sb->queue)) != NULL)
            mt_message_sbhandler(u,jq->jp);
    }
    else
    {
        log_debug(ZONE,"SB session, ANS error, %s",cmd);
        mt_sb_close(sb);
    }
    return 1;
}

void mt_phandle_sb_msg(mpacket ph, sbsession sb)
{
    session s = sb->s;
    jid from;
    xmlnode x, msg = NULL;
    char *body, *ctype;

    if (mt_parse_msg(ph,&body,&ctype) == 0)
    {
        log_debug(ZONE,"SB session, failed to parse message");
        return;
    }

    from = mt_mid2jid(ph->p,mt_packet_data(ph,1),s->server);

    if (j_strcmp(ctype,"text/x-msmsgscontrol") == 0)
    {
        return; /* XXX ignore until clients support message events */
        msg = mt_jmessage(JPACKET__NONE,jid_full(s->id),jid_full(from),NULL,sb->thread);
        x = xmlnode_insert_tag(msg,"x");
        xmlnode_put_attrib(x,"xmlns","jabber:x:event");
        xmlnode_insert_tag(x,"composing");
    }
    else if (j_strcmp(ctype,"text/plain") == 0)
    {
        body = mt_str_to_UTF8(ph->p,(unsigned char *) msg);
    }
    else if (j_strcmp(ctype,"text/plain; charset=UTF-8") != 0)
        return;

    if (msg == NULL)
        msg = mt_jmessage(JPACKET__CHAT,jid_full(s->id),jid_full(from),body,sb->thread);

    mt_deliver(s->ti,msg);
}
