/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * -------------------------------------------------------------------------- */

#include "msntrans.h"

void mt_s10n(session s, jpacket jp)
{
    muser u;
    pool p = jp->p;
    char *user, *h;

    if (s->connected == 0)
    {
        jpq jq;

        jq = pmalloco(p,sizeof(_jpq));
        jq->jp = jp;
        pth_msgport_put(s->queue,(void *) jq);
        return;
    }

    log_debug(ZONE,"Session[%s], handling subscription",jid_full(s->id));

    user =  mt_jid2mid(p,jp->to);
    if (user == NULL)
    {
        if (jp->to->user != NULL)
        {
            jutil_error(jp->x,(terror){406,"Invalid User"});
            mt_deliver(s->ti,jp->x);
        }
        else
            xmlnode_free(jp->x);
        return;
    }

    u = mt_muser(s,user);
    h = pstrdup(p,user);
    *(strchr(h,'@')) = '\0';  /* use their username as the handle */

    switch (jpacket_subtype(jp))
    {
    case JPACKET__SUBSCRIBE:
        if (u->fl == 0)
            mt_mpstream_rwrite(s->st,mt_phandle_add,mt_cmd_add(s->st->trid,"FL",user,h),(void *) s);

        /* if they are on the BL remove them */
        if (u->bl == 1)
            mt_mpstream_rwrite(s->st,mt_phandle_rem,mt_cmd_rem(s->st->trid,"BL",user),(void *) s);

        mt_deliver(s->ti,mt_presnew(JPACKET__SUBSCRIBED,jid_full(jp->from),jid_full(jp->to),NULL));
        mt_deliver(s->ti,mt_user_presnew(s,u->m,u->state));
        break;

    case JPACKET__SUBSCRIBED:
        if (u->al == 0)
            mt_mpstream_rwrite(s->st,mt_phandle_add,mt_cmd_add(s->st->trid,"AL",user,h),(void *) s);

        if (u->bl == 1)
            mt_mpstream_rwrite(s->st,mt_phandle_rem,mt_cmd_rem(s->st->trid,"BL",user),(void *) s);
        break;

    case JPACKET__UNSUBSCRIBE:
        if (u->fl == 1)
            mt_mpstream_rwrite(s->st,mt_phandle_rem,mt_cmd_rem(s->st->trid,"FL",user),(void *) s);

        mt_deliver(s->ti,mt_presnew(JPACKET__UNSUBSCRIBED,jid_full(jp->from),jid_full(jp->to),NULL));
        break;

    case JPACKET__UNSUBSCRIBED:
        if (u->al == 1)
            mt_mpstream_rwrite(s->st,mt_phandle_rem,mt_cmd_rem(s->st->trid,"AL",user),(void *) s);        
        break;
    }

    xmlnode_free(jp->x);
}

void mt_presence(session s, jpacket jp)
{
    muser u;
    pool p = jp->p;
    char *state;

    log_debug(ZONE,"Session[%s], handling presence",jid_full(s->id));

    switch (jpacket_subtype(jp))
    {
    case JPACKET__PROBE:
        u = (muser) ghash_get(s->muser_tab,mt_jid2mid(p,jp->to));
        if (u != NULL)
            mt_deliver(s->ti,mt_user_presnew(s,u->m,u->state));
        else
            mt_deliver(s->ti,mt_presnew(JPACKET__UNAVAILABLE,jid_full(jp->from),jid_full(jp->to),NULL));

        xmlnode_free(jp->x);
        break;

    case JPACKET__AVAILABLE:
        /* ignore presence sent to users */
        if (jp->to->user != NULL)
        {
            xmlnode_free(jp->x);
            return;
        }

        /* update users presence */
        s->p_db = ppdb_insert(s->p_db,jp->from,jp->x);
        state = mt_show2state(p,xmlnode_get_tag_data(jp->x,"show"));

        if (s->connected == 1 && mt_state2ustate(state) != s->state)
            mt_mpstream_rwrite(s->st,(handle) mt_phandle_chg,mt_cmd_chg(s->st->trid,state),(void*) s);

        s->state = mt_state2ustate(state);

        jutil_tofrom(jp->x);
        xmlnode_hide(xmlnode_get_tag(jp->x,"status"));
        xmlnode_insert_cdata(xmlnode_insert_tag(jp->x,"status"),s->connected ? "Connected" : "Connecting...",-1);
        mt_deliver(s->ti,jp->x);
        break;

    case JPACKET__UNAVAILABLE:        
        if (jp->to->user == NULL && xmlnode_get_tag(jp->x,"x") == NULL)
        {
            s->p_db = ppdb_insert(s->p_db,jp->from,jp->x);
            if (ppdb_primary(s->p_db,s->id) == NULL) /* Are there any resources left? */
                mt_session_end(s);
        }
    default:
        xmlnode_free(jp->x);
        break;
    }
}
