/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * -------------------------------------------------------------------------- */

#include "msntrans.h"

sbsession mt_sb_create(session s, muser u)
{
    sbsession sb;
    pool p;

    log_debug(ZONE,"creating SB session, %s",jid_full(s->id));

    p = pool_new();
    sb = pmalloco(p,sizeof(_sbsession));
    sb->p = p;
    sb->s = s;
    sb->queue = pth_msgport_create("mt_sb_queue");
    sb->connected = 0;
    sb->thread = pstrdup(p,shahash(u->m));

    mt_sb_add(sb,u);

    return sb;
}

void mt_sb_add(sbsession sb, muser u)
{
    sbuser cur, sbu;

    if (u == NULL)
        return;

    if (u->sb != NULL)
    {
        if (u->sb == sb)
        {
            log_debug(ZONE,"SB session, user '%s' already added",u->m);
            return;
        }

        log_debug(ZONE,"SB session replaced");
        /* we don't support multiple sessions with the same user */
        mt_sb_close(u->sb);
    }

    /* add them to the SB session */
    sbu = pmalloco(sb->p,sizeof(_sbuser));
    sbu->u = u;
    if (sb->users)
    {
        for (cur = sb->users; cur->next != NULL; cur = cur->next);
        cur->next = sbu;
    }
    else
        sb->users = sbu;

    u->sb = sb;

    log_debug(ZONE,"SB session, user '%s' joined, total %d",u->m,sb->count);
}

void mt_sb_bye(sbsession sb, char *m)
{
    muser u;
    sbuser cur, prev;

    log_debug(ZONE,"SB session, user '%s' left",m);

    /* if no one is left close the session */
    if (--sb->count <= 1)
    {
        mt_sb_close(sb);
        return;
    }

    u = mt_muser(sb->s,m);

    /* remove them from the SB session */
    for (prev = NULL, cur = sb->users;  cur != NULL; prev = cur, cur = cur->next)
    {
        if (u == cur->u)
        {
            cur->u->sb = NULL;
            if (prev)
                prev->next = cur->next;
            else
                sb->users = cur->next;
            break;
        }
    }

    log_debug(ZONE,"SB session, count %d",sb->count);
}

/* mpstream pool cleanup */
void mt_sb_cleanup(void *arg)
{
    sbsession sb = (sbsession) arg;
    sb->st = NULL;
    mt_sb_close(sb);
}

mpstream mt_sb_connect(sbsession sb, char *host, int port)
{
    mpstream st;

    sb->st = st = mt_mpstream_connect(sb->s,host,port,mt_phandle_sb_default,(void *) sb);
    pool_cleanup(st->p,mt_sb_cleanup,(void*) sb);

    return st;
}

void mt_sb_close(sbsession sb)
{
    session s;
    sbuser cur;

    if (sb == NULL)
        return;

    log_debug(ZONE,"SB close %X",sb);

    if (sb->users)
    {
        for (cur = sb->users;  cur != NULL; cur = cur->next) cur->u->sb = NULL;
        sb->users = NULL;
    }

    if (sb->st == NULL)
    {
        jpq jq;

        log_debug(ZONE,"freeing SB sesison");

        s = sb->s;
        while ((jq = (jpq) pth_msgport_get(sb->queue)) != NULL)
        {
            xmlnode x = jq->jp->x;

            if (s->exit_flag) {
                /* they don't have a session anymore */
                jutil_error(x,TERROR_NOTFOUND);
            } else {
                /* blam it on the MSN server */
                jutil_error(x,TERROR_EXTERNAL);
            }
            mt_deliver(s->ti,x);
        }

        pth_msgport_destroy(sb->queue);
        pool_free(sb->p);
    }
    else if (sb->connected != -1)
    {
        log_debug(ZONE,"SB session closing mio");
        sb->connected = -1;

        if (sb->st->c != NULL)
            mio_close(sb->st->c);
    }
}
