/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 */
#include "msntrans.h"
#include <sys/utsname.h>

void mt_unknown_iq(mti ti, jpacket jp);
void mt_unknown_presence(void *arg);
void mt_unknown_reg_get(mti ti, jpacket jp);
void mt_unknown_reg_set(mti ti, jpacket jp);
void mt_unknown_bounce(void *arg);

void mt_unknown(mti ti, jpacket jp)
{
    log_debug(ZONE,"Packet from unknown user '%s'",jid_full(jp->from));

    switch (jp->type)
    {
    case JPACKET_IQ:
        mt_unknown_iq(ti,jp);
        break;

    case JPACKET_MESSAGE:
    case JPACKET_S10N:
        jp->aux1 = (void *) ti;
        mtq_send(NULL,jp->p,mt_unknown_bounce,(void *) jp);
        break;

    case JPACKET_PRESENCE:
        if (jpacket_subtype(jp) == JPACKET__AVAILABLE && jp->to->user == NULL &&
            j_strncmp(jp->to->resource,"registered",10) == 0) {

            jp->aux1 = (void *) ti;
            mtq_send(NULL,jp->p,mt_unknown_presence,(void *) jp);
            break;
        }
    default:
        xmlnode_free(jp->x);
    }
}

void mt_unknown_iq(mti ti, jpacket jp)
{
    char *ns;

    if (jp->to->user != NULL)
    {
        jp->aux1 = (void *) ti;
        mtq_send(NULL,jp->p,mt_unknown_bounce,(void *) jp);
        return;
    }

    ns = xmlnode_get_attrib(jp->iq,"xmlns");

    switch (jpacket_subtype(jp))
    {
    case JPACKET__SET:
        if (j_strcmp(ns,NS_REGISTER) == 0)
            mt_unknown_reg_set(ti,jp);
        else
        {
            jutil_error(jp->x,TERROR_NOTALLOWED);
            mt_deliver(ti,jp->x);
        }
        break;

    case JPACKET__GET:
        if (j_strcmp(ns,NS_REGISTER) == 0)
            mt_unknown_reg_get(ti,jp);
        else if (j_strcmp(ns,NS_BROWSE) == 0)
            mt_iq_browse_server(ti,jp);
        else if (j_strcmp(ns,NS_VERSION) == 0)
            mt_iq_version(ti,jp);
        else if (j_strcmp(ns,NS_TIME) == 0)
            mt_iq_time(ti,jp);
        else if (j_strcmp(ns,NS_VCARD) == 0)
            mt_iq_vcard_server(ti,jp);
        else if (j_strcmp(ns,NS_LAST) == 0)
        {
            xmlnode_insert_node(jutil_iqresult(jp->x),ti->last);
            mt_deliver(ti,jp->x);
        }
        break;

    default:
        jutil_error(jp->x,TERROR_NOTALLOWED);
        mt_deliver(ti,jp->x);
    }
}

void mt_unknown_presence(void *arg)
{
    jpacket jp = (jpacket) arg;
    mti ti = jp->aux1;
    xmlnode reg;
    pool p = jp->p;
    session s;

    /* get account info from xdb */
    reg = xdb_get(ti->xc,mt_xdb_id(p,jp->from,jp->to->server),NS_REGISTER);
    if (reg == NULL)
    {
        jutil_error(jp->x,TERROR_REGISTER);
        mt_deliver(ti,jp->x);
        return;
    }

    s = mt_session_create(ti,jp);

    s->user = pstrdup(s->p,xmlnode_get_tag_data(reg,"username"));
    s->nick = mt_safe_nick(s->p,xmlnode_get_tag_data(reg,"nick"));
    s->pass = pstrdup(s->p,xmlnode_get_tag_data(reg,"password"));

    xmlnode_free(reg);

    s->muser_tab = ghash_create(25,(KEYHASHFUNC) str_hash_code,(KEYCOMPAREFUNC) j_strcmp);
    s->p_db = ppdb_insert(s->p_db,jp->from,jp->x);
    s->state = mt_state2ustate(mt_show2state(p,xmlnode_get_tag_data(jp->x,"show")));

    log_debug(ZONE,"Session[%s], starting",jid_full(s->id));

    mt_session_start(s);

    /* Echo back the users presence */
    jutil_tofrom(jp->x);
    xmlnode_hide(xmlnode_get_tag(jp->x,"status"));
    xmlnode_insert_cdata(xmlnode_insert_tag(jp->x,"status"),"Connecting...",-1);
    mt_deliver(ti,jp->x);
}

void mt_unknown_reg_get(mti ti, jpacket jp)
{
    xmlnode q;

    jutil_iqresult(jp->x);

    q = xmlnode_insert_tag(jp->x,"query");
    xmlnode_put_attrib(q,"xmlns",NS_REGISTER);

    xmlnode_insert_tag(q,"username");
    xmlnode_insert_tag(q,"password");
    xmlnode_insert_tag(q,"nick");

    xmlnode_insert_cdata(xmlnode_insert_tag(q,"instructions"),ti->reg,-1);
    xmlnode_insert_cdata(xmlnode_insert_tag(q,"key"),jutil_regkey(NULL,jid_full(jp->from)),-1);

    mt_deliver(ti,jp->x);
}

void mt_unknown_reg_set(mti ti, jpacket jp)
{
    session s;
    xmlnode q = jp->iq;
    jpq jq;
    char *user, *pass, *key;

    key = xmlnode_get_tag_data(q,"key");
    if (key == NULL || !jutil_regkey(key,jid_full(jp->from)))
    {
        jutil_error(jp->x,(terror){406,"Invalid key"});
        mt_deliver(ti,jp->x);
        return;
    }

    user = xmlnode_get_tag_data(jp->iq,"username");
    pass = xmlnode_get_tag_data(jp->iq,"password");

    /* Make sure it is a valid register query */
    if (user == NULL || pass == NULL)
    {
        jutil_error(jp->x,TERROR_BAD);
        mt_deliver(ti,jp->x);
        return;
    }

    s = mt_session_create(ti,jp);

    /* requiered fields, we don't  need to  check for nickname since it is not requiered */
    s->user = mt_safe_user(s->p,user);
    s->pass = pstrdup(s->p,pass);
    s->nick = mt_safe_nick(s->p,xmlnode_get_tag_data(jp->iq,"nick"));

    log_debug(ZONE,"Starting register session[%s]",jid_full(s->id));

    s->type = stype_register;

    jq = pmalloco(jp->p,sizeof(_jpq));
    jq->jp = jp;
    pth_msgport_put(s->queue,(void *) jq);

    mt_session_start(s);
}

/* bounces packet for unknown users with the appropriate error */
void mt_unknown_bounce(void *arg)
{
    jpacket jp = (jpacket) arg;
    mti ti = (mti) jp->aux1;
    xmlnode reg;

    if ((reg = xdb_get(ti->xc,mt_xdb_id(jp->p,jp->from,jp->to->server),NS_REGISTER)) != NULL)
    {
        jutil_error(jp->x,TERROR_NOTFOUND);
        xmlnode_free(reg);
    }
    else
        jutil_error(jp->x,TERROR_REGISTER);

    mt_deliver(ti,jp->x);
}
