/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * -------------------------------------------------------------------------- */

#include "msntrans.h"

xmlnode mt_presnew(int type, char *to, char *from, char *show)
{
    xmlnode x;

    x = jutil_presnew(type,to,NULL);

    if (from)
        xmlnode_put_attrib(x,"from",from);

    if (show)
        xmlnode_insert_cdata(xmlnode_insert_tag(x,"show"),show,-1);

    return x;
}

xmlnode mt_jmessage(int type, char *to, char *from, char *body, char *thread)
{
    xmlnode x;

    x = xmlnode_new_tag("message");

    xmlnode_put_attrib(x,"to",to);
    xmlnode_put_attrib(x,"from",from);

    if (type == JPACKET__CHAT)
        xmlnode_put_attrib(x,"type","chat");
    else if (type == JPACKET__HEADLINE)
        xmlnode_put_attrib(x,"type","headline");

    if (thread)
        xmlnode_insert_cdata(xmlnode_insert_tag(x,"thread"),thread,-1);

    if (body)
        xmlnode_insert_cdata(xmlnode_insert_tag(x,"body"),body,-1);

    return x;
}

char *mt_safe_user(pool p, char *user)
{
    char *str;
    int flag = 0;

    if (j_strlen(user) < 1)
        return NULL;

    for (str = user; *str != '\0'; str++)
    {
        if (*str <= 32 || *str == '\r' || *str == '\n'||
            *str == ':' || *str == '<' || *str == '>' ||
            *str == '\'' || *str == '"' || *str == '&')
            return NULL;
        if (*str == '@')
            flag++;
    }

    return flag == 1 ? pstrdup(p,user) : NULL;
}

char *mt_safe_nick(pool p, char *nick)
{
    spool sp = NULL;
    char *buf = nick;
    char *str;

    if (j_strlen(nick) < 1)
        return NULL;

    for (str = nick; *str != '\0'; str++)
    {
        if (*str == '\r' || *str == '\n'||*str == ':' ||
            *str == '@' || *str == '<' || *str == '>' ||
            *str == '\'' || *str == '"' || *str == '&')
            return NULL;

        /* escape spaces */
        if (*str == ' ')
        {
            if (sp == NULL)
                sp = spool_new(p);

            *str = '\0';
            spooler(sp,buf,"%20",sp);
            buf = ++str;
        }
    }

    if (sp == NULL)
        return pstrdup(p,nick);

    spool_add(sp,buf);
    return spool_print(sp);
}

jid mt_mid2jid(pool p, char *user, char *server)
{
    jid id;
    char *buf, *ptr;

    id = jid_new(p,server);
    id->user = buf = pstrdup(p,user);

    if ((ptr = strchr(buf,'@')) == NULL)
        return NULL;

    *ptr = '%';

    return id;
}

/* converts jid to an mid.  jids returned are in the form account%hotmail.com@domain */
char *mt_jid2mid(pool p, jid id)
{
    char *user, *ptr;

    if (id == NULL)
        return NULL;

    if ((user = pstrdup(p,id->user)) == NULL)
        return NULL;

    if ((ptr = strchr(user,'%')) == NULL)
        return NULL;

    *ptr = '@';

    return user;
}

xmlnode mt_user_presnew(session s, char *m, ustate state)
{
    xmlnode x, show;
    jid from;

    x = xmlnode_new_tag("presence");
    if (m)
    {
        from = mt_mid2jid(xmlnode_pool(x),m,s->server);
        xmlnode_put_attrib(x,"from",jid_full(from));
    }
    xmlnode_put_attrib(x,"to",jid_full(s->id));

    if (state != ustate_fln)
    {
        if (state != ustate_nln)
        {
            show = xmlnode_insert_tag(x,"show");
            switch (state)
            {
            case ustate_bsy:
                xmlnode_insert_cdata(show,"dnd",-1);
                break;
            case ustate_awy:
            case ustate_idl:
                xmlnode_insert_cdata(show,"xa",-1);
                break;
            case ustate_brb:
            case ustate_phn:
            case ustate_lun:
                xmlnode_insert_cdata(show,"away",-1);
                break;
            default:
                break;
            }
        }
    }
    else
        xmlnode_put_attrib(x,"type","unavailable");

    return x;
}

char *mt_ustate2state(pool p, ustate state)
{
    char *ret = NULL;

    switch (state)
    {
    case ustate_bsy:		/* Busy */
        ret = pstrdup(p,"BSY");
        break;
    case ustate_awy:		/* Away */
        ret = pstrdup(p,"AWY");
        break;
    case ustate_idl:		/* Idle */
        ret = pstrdup(p,"IDL");
        break;
    case ustate_brb:		/* Be Right Back */
        ret = pstrdup(p,"BRB");
        break;
    case ustate_phn:		/* On the phone */
        ret = pstrdup(p,"PHN");
        break;
    case ustate_lun:		/* Out to Lunch */
        ret = pstrdup(p,"LUN");
        break;
    case ustate_nln:
        ret = pstrdup(p,"NLN");
        break;
    case ustate_fln:
        ret = NULL;
    }

    return ret;
}

char *mt_show2state(pool p, char *show)
{
    char *ret;

    if (show == NULL)
        return pstrdup(p,"NLN");

    if (j_strcmp(show,"away") == 0)
        ret = pstrdup(p,"BRB");
    else if (j_strcmp(show,"xa") == 0)
        ret = pstrdup(p,"AWY");
    else if (j_strcmp(show,"dnd") == 0)
        ret = pstrdup(p,"BSY");
    else
        ret = pstrdup(p,"NLN");

    return ret;
}

ustate mt_state2ustate(char *state)
{
    ustate u;

    if (j_strcmp(state,"NLN") == 0)
        u = ustate_nln;
    else if (j_strcmp(state,"BSY") == 0)
        u = ustate_bsy;
    else if (j_strcmp(state,"IDL") == 0)
        u = ustate_idl;
    else if (j_strcmp(state,"BRB") == 0)
        u = ustate_brb;
    else if (j_strcmp(state,"AWY") == 0)
        u = ustate_awy;
    else if (j_strcmp(state,"PHN") == 0)
        u = ustate_phn;
    else if (j_strcmp(state,"LUN") == 0)
        u = ustate_lun;
    else
        u = ustate_fln;

    return u;
}

char *mt_fnd2cdata(char *fnd)
{
    char *ptr;

    if (fnd == NULL || (ptr = strchr(fnd,'=')) == NULL) return NULL;
    ++ptr;
    return j_strcmp(ptr,"*") == 0 ? NULL : ptr;
}

char *mt_ul2str(pool p, unsigned long ul)
{
    char buf[11];

    memset(buf,0,sizeof(buf));
    snprintf(buf,10,"%ld",ul);

    return pstrdup(p,buf);
}

int mt_parse_msg(mpacket ph, char **msg, char **ctype)
{
    char *ptr, *data, *ct;

    data = mt_packet_data(ph,4);

    /* MIME Version */
    ptr = strstr(data,"\r\n");
    if (ptr == NULL)
        return 0;

    *ptr = '\0';
    data = ptr += 2;

    /* Content type */
    ptr = strstr(data,"\r\n\r\n");
    if (ptr == NULL)
        return 0;

    *ptr = '\0';
    ptr += 4;
    ct = data;
    data = ptr;

    if (j_strlen(ptr) < 1)
        return 0;

    ptr = strchr(ct,':');
    if (ptr == NULL)
        return 0;

    *ptr = '\0';
    ct = ptr += 2;

    if ((ptr = strstr(ct,"\r")) != NULL)
        *ptr = '\0';

    *msg = data;
    *ctype = ct;

    return 1;
}

char *mt_str_to_UTF8(pool p, unsigned char *in)
{
    int n, i = 0;
    char *result = NULL;

    result = (char *) pmalloco(p,strlen(in) * 2 + 1);

    /* convert a string to UTF-8 Format */
    for (n = 0; n < strlen(in); n++)
    {
        long c = (long) in[n];

        if (c < 128)
        {
            result[i++] = (char) c;
        }
        else
        {
            result[i++] = (char) ((c >> 6) | 192);
            result[i++] = (char) ((c & 63) | 128);
        }
    }
    result[i] = '\0';
    return result;
}

jid mt_xdb_id(pool p, jid id, char *server)
{
    char *buf;
    buf = spools(p,id->user,"%",id->server,"@",server,p);
    return jid_new(p,buf);
}
